//
// Created by doggo on 04/02/2021.
//

#include "../headers/lex.hpp"
#include <iostream>
using namespace std;

vector<tok> resVal;

void separate() {resVal.emplace_back(tok::SEPARATOR, "");}

bool longLex = false;

tok::type getIdentifierCharType(const char& Char) {
    if (isalpha(Char)) return tok::IDENTIFIER;
    else if (isdigit(Char)) return tok::NUMBER;
    else if (Char == '"') return tok::STRING;
    else return tok::UNEXPECTED;
}

vector<tok>& lex(const string& in)
{
    pair<tok::type, string> generated;
    for (const char& current : in) {
        if (!longLex) {
            if (!generated.second.empty()) {
                resVal.emplace_back(generated.first, generated.second);
                generated.second = "";
            }
            switch (current) {
                case '#': resVal.emplace_back(tok::TAG, "#"); break;
                case '=': resVal.emplace_back(tok::DEFINE, "="); break;
                case '(': resVal.emplace_back(tok::LPAR, "("); break;
                case ')': resVal.emplace_back(tok::RPAR, ")"); break;
                case '{': resVal.emplace_back(tok::LBRACE, "{"); break;
                case '}': resVal.emplace_back(tok::RBRACE, "}"); break;
                case '[': resVal.emplace_back(tok::LBRACKET, "["); break;
                case ']': resVal.emplace_back(tok::RBRACKET, "]"); break;
                case '+': resVal.emplace_back(tok::PLUS, "+"); break;
                case '-': resVal.emplace_back(tok::HYPHEN, "-"); break;
                case '>': resVal.emplace_back(tok::LCOMP, ">"); break;
                case '<': resVal.emplace_back(tok::RCOMP, "<"); break;
                case '\'': resVal.emplace_back(tok::SQUOTE, "\'"); break;
                case ' ': case '\t': case '\r':
                case '\n': separate(); break;
                default: {
                    tok::type type = getIdentifierCharType(current);
                    switch (type) {
                        case tok::UNEXPECTED: break;
                        case tok::IDENTIFIER: case tok::NUMBER: {
                            generated.second.append(string(1, current));
                        }
                        case tok::STRING: {
                            generated.first = type;
                            longLex = true;
                        }; break;
                        default: resVal.emplace_back(type, string(1, current));
                    }
                    break;
                }
            }
        } else {
            tok::type currentType = getIdentifierCharType(current);
            if ((currentType == tok::STRING) || ((currentType == tok::UNEXPECTED) && ((generated.first == tok::IDENTIFIER) || (generated.first == tok::NUMBER)))) {
                longLex = false;
            } else generated.second.append(string(1, current));
        }
    }
    /* Test
    for (const auto& it : resVal) {
        if (it.toktype == tok::STRING) cout << '"' << it.toktext << '"' << '\n';
        if (it.toktype == tok::NUMBER) cout << it.toktext << "= number\n";
        if (it.toktype == tok::IDENTIFIER) cout << it.toktext << '\n';
    }
    */
    return resVal;
}